"""Workspaces package.

See https://github.com/BLSQ/openhexa/wiki/User-manual#about-workspaces for more information about OpenHEXA workspaces.
"""

from .workspace import workspace

__all__ = ["workspace"]
